/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GUITONE_H
#define GUITONE_H

#include "MainWindow.h"
#include "MonotoneManager.h"
#include "DebugLog.h"
#include "OpenPrompt.h"

#ifndef Q_WS_MACX
#include "ApplicationUpdate.h"
#endif

#include <QApplication>
#include <QList>
#include <QMap>
#include <QMutex>

class Monotone;

class Guitone : public QApplication
{
    Q_OBJECT

public:
    Guitone(int &, char**);
    ~Guitone();
    inline MonotoneManager * manager() { return monotoneManager; }

signals:
    void updateWindowList(const QStringList &);
    void updateRecentLists();
    void monotoneBinaryPathSet();

public slots:
    void checkForApplicationUpdates(bool silent = false);

    void loadWorkspace(const QString &);
    void loadDatabase(const QString &);

    void activateWindow(int);
    void raiseAllWindows();

    void quit();

private slots:
    void windowClosed(MainWindow *);
    void setMonotoneBinaryPath();
    void loadSomething();

private:
    bool tryLoadSomething(const QString &);
    void loadFromPath(const QString &);
    void triggerUpdateWindowList();
    void ensureCascadedWindowPlacement(MainWindow *);
    bool closeAllWindows();
    bool event(QEvent *);

    OpenPrompt * openPrompt;
    MonotoneManager * monotoneManager;
    QList<MainWindow *> openWindows;
    QMutex lock;
    bool somethingLoaded;

#ifndef Q_WS_MACX
    ApplicationUpdate * updateDialog;
#endif
};

#endif

