/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "AutomateCommand.h"
#include "Guitone.h"

AutomateCommand::AutomateCommand(AutomateCommandHelper * helper)
{
    if (helper == 0)
    {
        helper = new AutomateCommandHelper(this);
    }
    cmdHelper = helper;
}

AutomateCommand::~AutomateCommand()
{
    delete cmdHelper;
}

void AutomateCommand::enqueueTask(MonotoneThread * thread, const MonotoneTask & task)
{
    QMutexLocker locker(&lock);

    int threadNumber = thread->getThreadNumber();
    // we want to avoid multiple thread connections
    if (!connectedThreads.contains(threadNumber))
    {
        QObject::connect(
            thread, SIGNAL(taskAborted(const MonotoneTask &)),
            cmdHelper, SLOT(taskAborted(const MonotoneTask &))
        );

        QObject::connect(
            thread, SIGNAL(taskFinished(const MonotoneTask &)),
            cmdHelper, SLOT(taskFinished(const MonotoneTask &))
        );

        connectedThreads.append(threadNumber);
    }

    if (!queuedCommands.contains(threadNumber))
    {
        queuedCommands.insert(threadNumber, QList<int>());
    }

    queuedCommands[threadNumber].append(
        thread->enqueueTask(task)
    );

}
void AutomateCommand::enqueueWorkspaceTask(const WorkspacePath & ws, const MonotoneTask & task)
{
    MonotoneThread * thread = APP->manager()->getThreadForWorkspace(ws);
    enqueueTask(thread, task);
}

void AutomateCommand::enqueueDatabaseTask(const DatabaseFile & db, const MonotoneTask & task)
{
    MonotoneThread * thread = APP->manager()->getThreadForDatabase(db);
    enqueueTask(thread, task);
}

AutomateCommandHelper::AutomateCommandHelper(AutomateCommand * cmd)
  : command(cmd)
{}

void AutomateCommandHelper::taskAborted(const MonotoneTask & task)
{
    // do sth more reasonable here in a subclass,
    // i.e. try to query the data again
    C(QString("Task %1 (%2) aborted: %3")
        .arg(task.getCommandNumber())
        .arg(QString::fromUtf8(task.getEncodedInput()))
        .arg(task.getOutputUtf8())
    );
}

void AutomateCommandHelper::taskFinished(const MonotoneTask & task)
{
    int threadNumber = task.getThreadNumber();
    int cmdNumber = task.getCommandNumber();

    if (!command->queuedCommands.contains(threadNumber) ||
        !command->queuedCommands[threadNumber].contains(cmdNumber))
    {
        return;
    }
    command->processTaskResult(task);
}

