/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef AUTOMATE_COMMAND_H
#define AUTOMATE_COMMAND_H

#include "MonotoneThread.h"

#include <QString>
#include <QList>
#include <QMutex>

class AutomateCommandHelper;

class AutomateCommand
{
public:
    AutomateCommand(AutomateCommandHelper * helper = 0);
    virtual ~AutomateCommand();
    friend class AutomateCommandHelper;

protected:
    virtual void processTaskResult(const MonotoneTask &) = 0;
    void enqueueWorkspaceTask(const WorkspacePath &, const MonotoneTask &);
    void enqueueDatabaseTask(const DatabaseFile &, const MonotoneTask &);

private:
    void enqueueTask(MonotoneThread * thread, const MonotoneTask &);

    QMutex lock;
    QList<int> connectedThreads;
    QMap<int, QList<int> > queuedCommands;
    AutomateCommandHelper * cmdHelper;
};

// Qt's class hierarchy doesn't support multiple inheritance, so we need
// a helper object which manages our signal/slot connections
class AutomateCommandHelper : public QObject
{
    Q_OBJECT
public:
    AutomateCommandHelper(AutomateCommand *);

protected slots:
    virtual void taskAborted(const MonotoneTask &);

    void taskFinished(const MonotoneTask &);

private:
    AutomateCommand * command;
};

#endif
