/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GET_FILE_H
#define GET_FILE_H

#include "AutomateCommand.h"
#include "DiffParser.h"

#include <QAbstractItemModel>

struct ContentLine {
    QString content;
    enum Marker {Unchanged, Added, Removed} marker;
    ContentLine() {}
    ContentLine(QString c, Marker m) : content(c), marker(m) {}
};

typedef QVector<ContentLine *> Content;

class GetFile : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    GetFile(QObject *, const DatabaseFile &);
    virtual ~GetFile();

    // needed Qt Model methods
    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;
    QModelIndex getNextGroup(const QModelIndex &, bool recurse = false);
    QModelIndex getPrevGroup(const QModelIndex &, bool recurse = false);

public slots:
    void readFileById(const QString &);
    void readFileByName(const QString &, const QString &);
    void applyDiff(Diff * diff);

signals:
    void fileRead();

private:
    void readFile(QStringList, QStringList);
    void processTaskResult(const MonotoneTask &);

    Content fileContents;
    DatabaseFile databaseFile;
};

#endif

