/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INVENTORY_H
#define INVENTORY_H

#include "AutomateCommand.h"
#include "InventoryItem.h"

#include <QAbstractItemModel>

class Inventory : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    Inventory(QObject *);
    ~Inventory();

    QMap<FileEntry, FileEntryList> findUnaccountedRenames();

    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;
    bool canFetchMore( const QModelIndex &) const;
    void fetchMore(const QModelIndex &);

    static const int QueryLevel;

public slots:
    void setWorkspacePath(const WorkspacePath &);
    void refresh();

signals:
    void nodeLoaded(const QModelIndex &);

private:
    void processTaskResult(const MonotoneTask &);
    QModelIndex indexFromItem(ModelItem *, int) const;
    void insertRowsRecursive(ModelItem *, const QMap<ModelItem *, QList<ModelItem *> > &);
    void removeRowsRecursive(const QModelIndex &, int, int);
    void readInventory(const QString &);

    ModelItem * rootItem;
    QString branchName;
    QMap<QString, InventoryItem *>  itemMap;
    WorkspacePath workspacePath;

private slots:
    void setWorkspacePath();

signals:
    void modelCreated();
    void invalidWorkspaceFormat(const QString &);
};

#endif

