/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INVENTORY_ITEM_H
#define INVENTORY_ITEM_H

#include "IconProvider.h"
#include "vocab.h"

#include <QVariant>

class ModelItem : public QObject
{
    Q_OBJECT
public:
    ModelItem(const QString & l = QString());
    ModelItem(const ModelItem *);
    virtual ~ModelItem();

    virtual void setLabel(const QString &);
    virtual QString getLabel() const;
    virtual QVariant data(int, int) const;

    void deleteAllChildren();
    void appendChild(ModelItem *);
    void removeChild(ModelItem *);
    void setChildren(QList<ModelItem *>);
    QList<ModelItem *> getChildren() const;
    void setParent(ModelItem *);
    ModelItem * parent() const;
    ModelItem * child(int) const;
    int childCount() const;
    int row() const;
    bool isRoot() const;

protected:
    ModelItem * parentItem;
    QList<ModelItem *> children;
    QString label;
};

class PseudoItem : public ModelItem
{
    Q_OBJECT
public:
    enum Type { Undefined = 1, CdUp};

    PseudoItem(const QString & l, Type t) : ModelItem(l), type(t)
    {}
    PseudoItem(const PseudoItem * other) : ModelItem(other), type(other->type)
    {}

    bool isCdUp() const { return type == CdUp; }

private:
    Type type;
};

class InventoryItem : public ModelItem
{
    Q_OBJECT
public:
    // "Undefined" is no state we get back from mtn, this is rather the
    // default for nodes which are not tracked by mtn, thus have no old_node
    // and no new_node stanza
    enum FileType { Undefined = 1, File, Directory, None };

    InventoryItem(const Stanza &);
    InventoryItem(const InventoryItem *);

    inline QString  getRenameSource() const { return old_path; }
    inline QString  getRenameTarget() const { return new_path; }
    inline FileType getFSType()       const { return fs_type; }
    inline FileType getOldType()      const { return old_type; }
    inline FileType getNewType()      const { return new_type; }
    inline QString  getPath()         const { return path; }
    inline int      getStatus()       const { return status; }
    inline bool     isDirectory()     const { return fs_type == Directory ||
                                                     old_type == Directory ||
                                                     new_type == Directory; }

    QString getRelativePath(const QString &) const;
    QString getFilename() const;
    QString getBaseDirectory() const;
    QString getLabel() const;
    QString getStatusString() const;
    QString getRenameInfo() const;

    bool hasStatus(int) const;
    bool hasNotStatus(int) const;

    bool isNewNode() const;
    bool isOldNode() const;
    bool isTracked() const;
    bool hasChanged() const;

    void setExpanded();
    bool isExpanded(int) const;

    int getStatusRecursive() const;
    bool hasChangedRecursive() const;

    QVariant data(int, int) const;

    static const int RenameSource;
    static const int RenameTarget;
    static const int Added;
    static const int Dropped;
    static const int Missing;
    static const int Ignored;
    static const int Known;
    static const int Unknown;
    static const int Invalid;
    static const int AttributesChanged;
    static const int ContentsChanged;

private:
    QString path;
    QString old_path;
    QString new_path;

    FileType fs_type;
    FileType old_type;
    FileType new_type;

    int status;
    bool expanded;
};

#endif
