/***************************************************************************
*   Copyright (C) 2006 by Thomas Keller                                   *
*   me@thomaskeller.biz                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "BasicIOParser.h"
#include "vocab.h"

BasicIOParser::BasicIOParser(const QString & input) : AbstractParser(input) {}

bool BasicIOParser::parse()
{
    while (true)
    {
        Stanza stanza = getStanza();
        if (stanza.empty()) break;
        stanzas.append(stanza);
        advance();
    }
    return true;
}

Stanza BasicIOParser::getStanza()
{
    Stanza stanza;
    
    while (true)
    {
        char next = whatsNext();
        if (next == '\n' || next == '\0') break;

        StanzaEntry entry;
        entry.sym = getSymbol();
        if (entry.sym.size() == 0)
        {
            W("Couldn't get symbol.");
        }
        QString hash(getHash());
        
        // was this a hash?
        if (!hash.isNull())
        {
            entry.hash = hash;
        }
        else
        {
            //grab all string values and put them into a list
            while (true)
            {
                // get value returns an empty string if there are no
                // more opening quotes
                QString value(getValue());
                if (value.isNull()) break;
                entry.vals.append(value);
            }
        }
        stanza.append(entry);
       
        eatSpaces();
        char cur = getNext();
        if (cur != '\n')
        {
            W(QString("Expected '\\n', got '%1'").arg(cur));
        }
    }
    
    return stanza;
}

QString BasicIOParser::getSymbol()
{
    eatSpaces();
    QByteArray payload;
    while (true)
    {
        char cur = whatsNext();
        if ((cur >= 'a' && cur <= 'z') || cur == '_')
        {
            payload.append(cur);
            advance();
            continue;
        }
        break;
    }
    
    // ascii conv is enough here
    return QString(payload);
}

QString BasicIOParser::getValue()
{
    eatSpaces();
    if (whatsNext() != '"') return QString();
    advance();
    
    QByteArray payload;
    char last = '\0';
    while (true)
    {
        char cur = getNext();
        I(cur != '\0');
        // string end?
        if (cur == '"' && last != '\\') break;
        last = cur;
        payload.append(cur);
    }
    
    QString ret = QString::fromUtf8(payload);
    ret.replace("\\\\", "\\");
    ret.replace("\\\"", "\"");
    return ret;
}

QString BasicIOParser::getHash()
{
    eatSpaces();
    if (whatsNext() != QChar('[')) return QString();
    advance();
    
    // special case: nonexistant revision ID
    if (whatsNext() == QChar(']'))
    {
        advance();
        return QString("");
    }
    
    int hashLen = 40;
    QString ret = QString::fromUtf8(input.mid(0, hashLen).data());
    advance(hashLen);
    
    char ch = whatsNext();
    if (ch == QChar(']'))
    {
        advance();
    }
    else
    {
        W(QString("Expected ], got '%1'").arg(ch));
    }
    return ret;
}

