/***************************************************************************
 *   Copyright (C) 2006 by Adam Higerd                                     *
 *   ahigerd@stratitec.com                                                 *
 *                                                                         *
 *   taken from http://qtnode.net/wiki/SignalWaiter                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "SignalWaiter.h"
#include "vocab.h"

#include <QCoreApplication>
#include <QTimerEvent>

SignalWaiter::SignalWaiter(const QObject* sender, const char* signal)
    : QObject(0) 
{
    I(sender && signal);
    connect(sender, signal, this, SLOT(signalCaught()));
}   
    
// Returns true if the signal was caught, returns false if the wait timed out
bool SignalWaiter::wait(const QObject* sender, const char* signal, int msec)
{
    SignalWaiter w(sender, signal);
    return w.wait(msec);
}

void SignalWaiter::signalCaught()
{
    ready = true;
}

void SignalWaiter::timerEvent(QTimerEvent* event)
{
    killTimer(timer);
    timeout = true;
}

// Returns true if the signal was caught, returns false if the wait timed out
bool SignalWaiter::wait(int msec)
{
    // Check input parameters
    if (msec < -1) return false;
    
    // activate the timeout
    if (msec != -1) timer = startTimer(msec);
 
    // Begin waiting   
    ready = timeout = false;
    while (!ready && !timeout)
        QCoreApplication::processEvents(QEventLoop::WaitForMoreEvents);
 
    // Clean up and return status
    killTimer(timer);
    return ready || !timeout;
}

