/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "DatabaseWindow.h"
#include "DatabaseMenuBar.h"
#include "DatabaseDialogManager.h"
#include "Guitone.h"
#include "Settings.h"

#include <QMessageBox>
#include <QHBoxLayout>

DatabaseWindow::DatabaseWindow() : MainWindow(), loadedDatabase(0)
{
    setObjectName("DatabaseWindow");
}

DatabaseWindow::~DatabaseWindow()
{
    if (loadedDatabase) delete loadedDatabase;
}

void DatabaseWindow::init()
{
    if (!menuBar)
    {
        menuBar = new DatabaseMenuBar(this);
    }

    if (!dialogManager)
    {
        dialogManager = new DatabaseDialogManager(this);
    }

    connect(
        dialogManager, SIGNAL(revisionCheckedOut(const QString &)),
        APP, SLOT(loadWorkspace(const QString &))
    );

    connect(
        menuBar, SIGNAL(showChangesetBrowser()),
        dialogManager, SLOT(showChangesetBrowser())
    );

    connect(
        menuBar, SIGNAL(showCheckoutRevision()),
        dialogManager, SLOT(showCheckoutRevision())
    );

    connect(
        menuBar, SIGNAL(showKeyManagement()),
        dialogManager, SLOT(showKeyManagement())
    );

    MainWindow::init();
}

void DatabaseWindow::setup()
{
    QWidget * centralWidget = new QWidget(this);
    QHBoxLayout * layout = new QHBoxLayout(centralWidget);

    loadedDatabase = new QLabel(tr("nothing loaded"), this);
    loadedDatabase->setTextFormat(Qt::PlainText);
    loadedDatabase->setAlignment(Qt::AlignCenter);

    layout->addWidget(loadedDatabase);

    setCentralWidget(centralWidget);
}

void DatabaseWindow::load(const QString & path)
{
    APP->manager()->getThreadForDatabase(path);

    databaseFile = path;

    // initialize the dialog manager
    reinterpret_cast<DatabaseDialogManager *>(dialogManager)->init(path);

    loadedDatabase->setText(tr("Loaded database: %1").arg(path));

    QFileInfo fi(path);
    setWindowTitle(
        tr("%1 - database mode - guitone").arg(fi.fileName())
    );

    Settings::addItemToList("RecentDatabaseList", path, 5);
}

