/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ApplicationUpdate.h"
#include "SignalWaiter.h"
#include "MonotoneManager.h"
#include "Platform.h"

#include <QHttp>
#include <QDomDocument>

ApplicationUpdate::ApplicationUpdate(QWidget * parent)
  : Dialog(parent), update_available(false)
{
    setupUi(this);
    Dialog::init();

    connect(
        visitWebsite, SIGNAL(clicked()),
        this, SLOT(openWebsite())
    );

    QHttp http;
    http.setHost("guitone.thomaskeller.biz");
    http.get("/web/appcast.xml");

    SignalWaiter waiter(&http, SIGNAL(done(bool)));

    // TODO: we definitely should display some status window here
    // while waiting for a response on slow connections
    waiter.wait();

    QDomDocument doc;
    QString err;
    int line, col;

    if (!doc.setContent(http.readAll(), true, &err, &line, &col))
    {
        W(QString("Cannot parse document: %1 (line %2, col %3)")
          .arg(err).arg(line).arg(col));
        return;
    }

    QDomElement channel = doc.documentElement().firstChild().toElement();
    I(!channel.isNull());

    QDomNodeList items = channel.elementsByTagName("item");

    QPair<QString, QString> latest(GUITONE_VERSION, "");

    for (int i=0, j=items.size(); i<j; i++)
    {
        QDomElement item = items.item(i).toElement();
        if (item.isNull()) continue;
        QDomNodeList enclosures = item.elementsByTagName("enclosure");

        for (int k=0, l=enclosures.size(); k<l; k++)
        {
            QDomElement enclosure = enclosures.item(k).toElement();
            if (enclosure.isNull()) continue;
            QString version = enclosure.attributeNS(
                "http://www.andymatuschak.org/xml-namespaces/sparkle",
                "version", "not-found"
            );

            // each item can have multiple enclosures, of which at least one
            // should have a version field
            if (version == "not-found") continue;

            if (MonotoneManager::versionCompare(version, latest.first) > 0)
            {
                latest.first = version;
                QDomNodeList descs = item.elementsByTagName("description");
                I(descs.size() == 1);
                QDomElement desc = descs.item(0).toElement();
                I(!desc.isNull());
                latest.second = desc.text();
            }
        }
    }

    if (latest.first != GUITONE_VERSION)
    {
        textBrowser->setHtml(latest.second);
        update_available = true;
    }
}

ApplicationUpdate::~ApplicationUpdate() {}

void ApplicationUpdate::openWebsite()
{
    Platform::openFile("http://guitone.thomaskeller.biz");
    accept();
}

