/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FileHistory.h"

FileHistory::FileHistory(QWidget * parent, const DatabaseFile & databaseFile)
  : Dialog(parent)
{
    setupUi(this);
    Dialog::init();

    splitter->restoreState();

    changeModel = new GetContentChanged(this, databaseFile);
    certsModel  = new Certs(this, databaseFile);

    // assign the models to the views
    revisionFileList->setModel(changeModel);
    certList->setModel(certsModel);

    // display the certs of a selected revision on click
    connect(
        revisionFileList, SIGNAL(clicked(const QModelIndex &)),
        this, SLOT(readCerts(const QModelIndex &))
    );

    connect(
        selectFirst, SIGNAL(clicked()),
        this, SLOT(setFirstRevision())
    );

    connect(
        selectSecond, SIGNAL(clicked()),
        this, SLOT(setSecondRevision())
    );

    connect(
        showDiff, SIGNAL(clicked()),
        this, SLOT(showDiffDialog())
    );

    connect(
        revisionFileList, SIGNAL(clicked(const QModelIndex &)),
        this, SLOT(revisionFileListClicked(const QModelIndex &))
    );

}

FileHistory::~FileHistory()
{
    splitter->saveState();
    delete changeModel;
    delete certsModel;
}

void FileHistory::readHistory(const QString & fileName, const QString & startRevision)
{
    setWindowTitle(tr("History of %1").arg(fileName));
    changeModel->readChanges(fileName, startRevision);
}

void FileHistory::readCerts(const QModelIndex & index)
{
    if (!index.isValid()) return;

    QModelIndex revIdx = changeModel->index(index.row(), 0, QModelIndex());
    QString rev(revIdx.data().toString());

    certsModel->readCerts(rev);
}

void FileHistory::setFirstRevision()
{
    QModelIndex revIdx = changeModel->index(curRow, 0, QModelIndex());
    QModelIndex fileIdx = changeModel->index(curRow, 1, QModelIndex());
    I(revIdx.isValid() && fileIdx.isValid());

    firstRevision = revIdx.data().toString();
    fileName = fileIdx.data().toString();

    if (secondRevision.size() > 0) showDiff->setEnabled(true);

    selectFirst->setText(tr("First: %1...").arg(firstRevision.left(12)));
}

void FileHistory::setSecondRevision()
{
    QModelIndex revIdx = changeModel->index(curRow, 0, QModelIndex());
    I(revIdx.isValid());

    secondRevision = revIdx.data().toString();

    if (firstRevision.size() > 0) showDiff->setEnabled(true);

    selectSecond->setText(tr("Second: %1...").arg(secondRevision.left(12)));
}

void FileHistory::revisionFileListClicked(const QModelIndex & index)
{
    if (!index.isValid()) return;

    curRow = index.row();

    // make sure both button are enabled
    selectFirst->setEnabled(true);
    selectSecond->setEnabled(true);
}

void FileHistory::showDiffDialog()
{
    I(firstRevision.size() > 0 && secondRevision.size() > 0);
    emit fileDiff(fileName, firstRevision, secondRevision);
}

