/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "UnaccountedRenames.h"
#include "IconProvider.h"

#include <QFont>
#include <QHeaderView>

UnaccountedRenames::UnaccountedRenames(QWidget * parent) : Dialog(parent)
{
    setupUi(this);
    Dialog::init();

    connect(
        renameTree, SIGNAL(itemClicked(QTreeWidgetItem *, int)),
        this, SLOT(itemClicked(QTreeWidgetItem *, int))
    );

    renameTree->setHeaderLabels(QStringList()
        << tr("Missing paths and possible rename targets")
        << tr("File ID (if applicable)")
    );

    renameTree->header()->resizeSection(0, 300);
}

UnaccountedRenames::~UnaccountedRenames() {}

void UnaccountedRenames::showUnaccountedRenames(const QMap<FileEntry, FileEntryList> & renames)
{
    renameTree->clear();

    QFont boldFont;
    boldFont.setBold(true);
    IconProvider * iconProvider = IconProvider::singleton();

    QMapIterator<FileEntry, FileEntryList> i(renames);
    while (i.hasNext())
    {
        i.next();
        FileEntry sen = i.key();
        FileEntryList list = i.value();
        QTreeWidgetItem * source = new QTreeWidgetItem(renameTree);
        source->setFlags(Qt::ItemIsEnabled);
        source->setText(0, QString("%1 (%2)").arg(sen.path).arg(list.size()));
        source->setData(0, Qt::UserRole, QVariant(sen.path));
        source->setFont(0, boldFont);
        source->setIcon(0, sen.is_dir ?
                        iconProvider->getPlainFolderIcon() :
                        iconProvider->getPlainFileIcon());
        source->setText(1, sen.fileid.isEmpty() ? tr("n/a") : sen.fileid);
        source->setFont(1, boldFont);

        foreach (FileEntry en, list)
        {
            QTreeWidgetItem * target = new QTreeWidgetItem(source);
            target->setFlags(Qt::ItemIsEnabled | Qt::ItemIsUserCheckable);
            target->setCheckState(0, Qt::Unchecked);
            target->setText(0, en.path);
            target->setToolTip(0, en.fileid.isEmpty() || sen.fileid != en.fileid ?
                               tr("matched by name") :
                               tr("matched by content"));

            target->setText(1, en.fileid.isEmpty() ? tr("n/a") : en.fileid);
        }
    }
}

void UnaccountedRenames::itemClicked(QTreeWidgetItem * item, int column)
{
    if (item == 0 || column == -1) return;
    QTreeWidgetItem * parent = item->parent();
    // top level item clicked
    if (!parent) return;

    Qt::CheckState state = item->checkState(0);

    for (int i=0, j=parent->childCount(); i<j; i++)
    {
        parent->child(i)->setCheckState(0, Qt::Unchecked);
    }

    item->setCheckState(0, state);
}

